<?php

/**
 *
 * @package CacheGBPages
 * @copyright (c) 2024 PPK
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 *
 *
 */

namespace ppk\cachegbpages\acp;

/**
 * @ignore
 */
if (!defined('IN_PHPBB'))
{
	exit;
}

/**
 * @package acp
 */
class cachegbpages_module
{
	public $u_action;

	public function __construct()
	{
		global $user, $template, $request, $config, $phpbb_root_path, $phpbb_admin_path, $phpEx, $cache;

		$this->user = $user;
		$this->template = $template;
		$this->request = $request;
		$this->config = $config;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->phpbb_admin_path = $phpbb_admin_path;
		$this->phpEx = $phpEx;
		$this->cache = $cache;

		$this->cache_dir = $phpbb_root_path . 'cache/'.PHPBB_ENVIRONMENT.'/';
		$this->_cache_size = 0;
		$this->_number_files = 0;

		$user->add_lang('acp/common');
		$user->add_lang_ext('ppk/cachegbpages', 'acp_cachegbpages');

	}

	function main($id, $mode)
	{
		global $config, $request, $template, $user, $acm_type, $cache;

		$this->tpl_name = 'acp_cachegbpages';
		$this->page_title = $user->lang('CACHEGBPAGES');

		$form_key = 'acp_cachegbpages';
		add_form_key($form_key);

		$submit = ($this->request->is_set_post('submit')) ? true : false;

		if ($this->calculate_cache_size() !== false)
		{
			$cache_size_msg = $user->lang('CGBP_CACHE_INFO',
				number_format($this->_number_files, 0, '.', $user->lang('CGBP_TH_SEP')),
				get_formatted_filesize($this->_cache_size));
		}
		else
		{
			$cache_size_msg = $user->lang('CGBP_CACHE_INFO_ERROR');
		}
		$cgbp_cache_last_gc=$config['cgbp_cache_last_gc'] ? ' ('.$user->lang['CGBP_LAST_GC'].': '.$user->format_date($config['cgbp_cache_last_gc']).')' : '';

		$display_vars = array(
			'title' => 'ACP_CACHEGBPAGES',
			'vars' => array(
				'legend1' => 'CACHEGBPAGES_CONFIG',
				'cgbp_enabled' => array('lang' => 'CGBP_ENABLED', 'validate' => 'int', 'type' => 'custom', 'function' => array($this, 'select_cgbp_target'), 'explain' => true),
				'cgbp_exclude' => array('lang' => 'CGBP_EXCLUDE', 'validate' => 'string', 'type' => 'custom', 'function' => array($this, 'select_cgbp_exclude'), 'explain' => true),
				'cgbp_ttl' => array('lang' => 'CGBP_TTL', 'validate' => 'int', 'type' => 'text:5:5', 'explain' => true/*, 'append'=>' '.$user->lang['MINUTES']*/),
				'cgbp_cache_gc' => array('lang' => 'CGBP_CACHE_GC', 'validate' => 'int', 'type' => 'text:5:5', 'explain' => true, 'append'=>$cgbp_cache_last_gc),
				'cgbp_remove_sid' => array('lang' => 'CGBP_REMOVE_SID', 'validate' => 'bool', 'type' => 'radio:yes_no', 'explain' => true),
				'cgbp_force_fcache' => array('lang' => 'CGBP_FORCE_FCACHE', 'validate' => 'bool', 'type' => 'radio:yes_no', 'explain' => true),
				'cgbp_update_topic' => array('lang' => 'CGBP_UPDATE_TOPIC', 'validate' => 'bool', 'type' => 'radio:yes_no', 'explain' => true),
				'cgbp_debug_info' => array('lang' => 'CGBP_DEBUG_INFO', 'validate' => 'bool', 'type' => 'radio:yes_no', 'explain' => true),
				'legend3' => 'ACP_SUBMIT_CHANGES',

				)
		);

		if (isset($display_vars['lang']))
		{
			$user->add_lang($display_vars['lang']);
		}

		$this->new_config = $config;
		$cfg_array = (isset($_REQUEST['config'])) ? utf8_normalize_nfc($this->request->variable('config', array('' => ''), true)) : $this->new_config;
		$error = array();
		$message = array();

		// check for cache type settings
		if ($acm_type != 'file' && $acm_type != 'phpbb\\cache\\driver\\file' && !$cfg_array['cgbp_force_fcache'])
		{
			$error[] = $user->lang('CGBP_CACHE_TYPE_ERROR', $acm_type);
		}

		// We validate the complete config if wished
		validate_config_vars($display_vars['vars'], $cfg_array, $error);

		if ($submit && !check_form_key($form_key))
		{
			$error[] = $user->lang['FORM_INVALID'];
		}

		// We go through the display_vars to make sure no one is trying to set variables he/she is not allowed to...
		foreach ($display_vars['vars'] as $config_name => $data)
		{
			if (!isset($cfg_array[$config_name]) || strpos($config_name, 'legend') !== false)
			{
				continue;
			}

			$this->new_config[$config_name] = $config_value = $cfg_array[$config_name];

			if($submit)
			{
				if (strpos($data['type'], 'password') === 0 && $config_value === '********')
				{
					// Do not update password fields if the content is ********,
					// because that is the password replacement we use to not
					// send the password to the output
					continue;
				}
			}

			if (!count($error))
			{
				$this->config->set($config_name, $config_value);
			}
		}

		// Do not write values if there is an error
		if (count($error))
		{
			$submit = false;
		}

		if ($submit)
		{
			$cgbp_exclude=$this->request->variable('cgbp_exclude', array(0=>''));
			$cgbp_exclude_value=implode(',', $cgbp_exclude);
			$this->config->set('cgbp_exclude', $cgbp_exclude_value);
			if(count($cgbp_exclude)==3)
			{
				$this->config->set('cgbp_enabled', 0);
				$this->new_config['cgbp_enabled']=0;
			}

			add_log('admin', 'LOG_CONFIG_CACHEGBPAGES');

			$message = (count($message) ? implode('<br />', $message).'<br /><br />' : '').$user->lang('CONFIG_UPDATED');
			$message_type = E_USER_NOTICE;

			trigger_error($message . adm_back_link($this->u_action), $message_type);
		}

		$this->page_title = $display_vars['title'];

		$template->assign_vars(array(
			'L_TITLE' => $user->lang[$display_vars['title']],
			'L_TITLE_EXPLAIN' => $user->lang[$display_vars['title'] . '_EXPLAIN'].'<br />'.$cache_size_msg,

			'S_ERROR' => (count($error)) ? true : false,
			'ERROR_MSG' => implode('<br />', $error),
			'CACHEGBPAGES_BOARD_URL' => $this->phpbb_root_path,
			'CACHEGBPAGES_PAGE' => true,

			'U_ACTION' => $this->u_action)
		);

		// Output relevant page
		foreach ($display_vars['vars'] as $config_key => $vars)
		{
			if (!is_array($vars) && strpos($config_key, 'legend') === false)
			{
				continue;
			}

			if (strpos($config_key, 'legend') !== false)
			{
				$template->assign_block_vars('options', array(
					'S_LEGEND' => true,
					'LEGEND' => (isset($user->lang[$vars])) ? $user->lang[$vars] : $vars)
				);

				continue;
			}

			$type = explode(':', $vars['type']);

			$l_explain = '';
			if ($vars['explain'] && isset($vars['lang_explain']))
			{
				$l_explain = (isset($user->lang[$vars['lang_explain']])) ? $user->lang[$vars['lang_explain']] : $vars['lang_explain'];
			}
			else if ($vars['explain'])
			{
				$l_explain = (isset($user->lang[$vars['lang'] . '_EXPLAIN'])) ? $user->lang[$vars['lang'] . '_EXPLAIN'] : '';
			}

			$content = build_cfg_template($type, $config_key, $this->new_config, $config_key, $vars);

			if (empty($content))
			{
				continue;
			}

			$template->assign_block_vars('options', array(
				'KEY' => $config_key,
				'TITLE' => (isset($user->lang[$vars['lang']])) ? $user->lang[$vars['lang']] : $vars['lang'],
				'S_EXPLAIN' => $vars['explain'],
				'TITLE_EXPLAIN' => $l_explain,
				'CONTENT' => $content,
				)
			);

			unset($display_vars['vars'][$config_key]);
		}

	}

	public  function calculate_cache_size()
	{
		$dir = @opendir($this->cache_dir);

		if (!$dir)
		{
			return false;
		}

		$this->_cache_size = 0;
		$this->_number_files = 0;

		while (($entry = readdir($dir)) !== false)
		{
			if (is_file($this->cache_dir . $entry) && strpos($entry, 'data_cgbp_')===0 && strpos($entry, '.lock')===false)
			{
				$this->_cache_size += filesize($this->cache_dir . $entry);
				$this->_number_files++;
			}
		}

		closedir($dir);

		return true;
	}

	public function select_cgbp_target($value, $key)
	{
		global $user;

		$cgbp_targets=array(0=>$user->lang['NO'], 3=>$user->lang['YES'], 1=>$user->lang['CGBP_GUEST_ONLY'], 2=>$user->lang['CGBP_BOTS_ONLY']);

		$select_target='<select name="config['.$key.']">';
		foreach($cgbp_targets as $k=>$v)
		{
			$select_target.='<option value="'.$k.'"'.($k==$value ? ' selected="selected"' : '').'>'.$v.'</option>';
		}

		$select_target.='</select>';

		return $select_target;
	}

	public function select_cgbp_exclude($value, $key)
	{
		global $user;

		$cgbp_exclude=array('index'=>$user->lang['CGBP_INDEX'], 'viewforum'=>$user->lang['CGBP_VIEWFORUM'], 'viewtopic'=>$user->lang['CGBP_VIEWTOPIC']);

		$values=explode(',', $value);
		$select_exclude='<select name="'.$key.'[]" multiple="multiple">';

		foreach($cgbp_exclude as $k=>$v)
		{
			$select_exclude.='<option value="'.$k.'"'.(in_array($k, $values) ? ' selected="selected"' : '').'>'.$v." ($k)</option>";
		}

		$select_exclude.='</select>';

		return $select_exclude;
	}
}

?>
