<?php

/**
*
* @package CacheGBPages
* @copyright (c) 2024 PPK
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*
*/

namespace ppk\cachegbpages\core;

/**
 * @ignore
 */
if (!defined('IN_PHPBB'))
{
	exit;
}

class cgbp_functions
{
	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\request\request_interface */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var string phpBB root path */
	protected $phpbb_root_path;

	/** @var string PHP extension */
	protected $phpEx;

	/** @var \phpbb\cache\service */
	protected $cache;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	public function __construct(\phpbb\config\config $config, \phpbb\request\request_interface $request, \phpbb\template\template $template, \phpbb\user $user, $phpbb_root_path, $phpEx, \phpbb\cache\service $cache, \phpbb\db\driver\driver_interface $db)
	{
		global $acm_type;

		$this->config = $config;
		$this->request = $request;
		$this->template = $template;
		$this->user = $user;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->phpEx = $phpEx;
		$this->cache = $cache;
		$this->db = $db;

		$this->cache_dir = $phpbb_root_path . 'cache/'.PHPBB_ENVIRONMENT.'/';
		$this->forumcache_destroyed = array();
		$this->topiccache_destroyed = array();
		$this->cgbp_exclude=$config['cgbp_exclude'] ? array_flip(explode(',', $config['cgbp_exclude'])) : array();

		if ($acm_type != 'file' && $acm_type != 'phpbb\\cache\\driver\\file' && $config['cgbp_force_fcache'])
		{
			// force file cache using
			if (!class_exists('acm_file_extended'))
			{
				include($phpbb_root_path . 'ext/ppk/cachegbpages/includes/acm/acm_file_extended.' . $phpEx);
			}

			if (class_exists('acm_file_extended'))
			{
				$this->cache = new \acm_file_extended();
			}
		}
	}

	public function get_topic_id($post_id)
	{
		$sql_array = array(
			'SELECT' => 't.*, f.*',

			'FROM' => array(FORUMS_TABLE => 'f'),
		);

		$sql_array['SELECT'] .= ', p.post_visibility, p.post_time, p.post_id';
		$sql_array['FROM'][POSTS_TABLE] = 'p';
		$sql_array['FROM'][TOPICS_TABLE] = 't';
		$sql_array['WHERE'] = "p.post_id = $post_id AND t.topic_id = p.topic_id";
		$sql_array['WHERE'] .= ' AND f.forum_id = t.forum_id';

		$sql = $this->db->sql_build_query('SELECT', $sql_array);
		$result = $this->db->sql_query($sql);
		$topic_data = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		if (!$topic_data)
		{
			trigger_error('NO_TOPIC');
		}

		$topic_id=	$topic_data['topic_id'];

		$sql = 'SELECT COUNT(p.post_id) AS prev_posts
			FROM ' . POSTS_TABLE . " p
			WHERE p.topic_id = $topic_id AND p.post_visibility = " . ITEM_APPROVED
				." AND (p.post_time < {$topic_data['post_time']} OR (p.post_time = {$topic_data['post_time']} AND p.post_id <= {$topic_data['post_id']}))";

		$result = $this->db->sql_query($sql);
		$row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);
		$topic_data['prev_posts'] = $row['prev_posts'] - 1;

		return $topic_data;
	}

	public function is_cacheable_page($page_name, $topic_id=0, $start=0)
	{
		$page_name=basename($page_name);

		if(!preg_match('/^(index|viewforum|viewtopic)\.'.$this->phpEx.'/', $page_name))
		{
			return false;
		}

		$page_ary=pathinfo($page_name);
		$page=$page_ary['filename'];

		if($this->config['cgbp_exclude'])
		{
			$cgbp_exclude=explode(',', $this->config['cgbp_exclude']);
			if(in_array($page, $cgbp_exclude))
			{
				return false;
			}
		}

		if($page=='index')
		{
			$page_name="_cgbp_$page";
		}
		else if($page=='viewforum')
		{
			$forum_id	= $this->request->variable('f', 0);
			$start		= $this->request->variable('start', 0);

			$default_sort_days	= !empty($this->user->data['user_topic_show_days']) ? $this->user->data['user_topic_show_days'] : 0;
			$default_sort_key	= !empty($this->user->data['user_topic_sortby_type']) ? $this->user->data['user_topic_sortby_type'] : 't';
			$default_sort_dir	= !empty($this->user->data['user_topic_sortby_dir']) ? $this->user->data['user_topic_sortby_dir'] : 'd';

			$sort_days	= $this->request->variable('st', $default_sort_days);
			$sort_key	= $this->request->variable('sk', $default_sort_key);
			$sort_dir	= $this->request->variable('sd', $default_sort_dir);
			if (
				$forum_id &&
				$sort_days == $default_sort_days &&
				$sort_key == $default_sort_key &&
				$sort_dir == $default_sort_dir)
			{
				$page_name="_cgbp_{$page}_f{$forum_id}_s{$start}";
			}
			else {
				return false;
			}
		}
		else if($page=='viewtopic')
		{
			$topic_id ? '' : $topic_id	= $this->request->variable('t', 0);

			if(!$topic_id)
			{
				$post_id=$this->request->variable('p', 0);

				if(!$post_id)
				{
					return false;
				}

				$topic_data=$this->get_topic_id($post_id);
				$topic_id=$topic_data['topic_id'];
				$start = floor(($topic_data['prev_posts']) / $this->config['posts_per_page']) * $this->config['posts_per_page'];
			}

			$view		= $this->request->variable('view', '');
			$update		= $this->request->variable('update', false);
			$start ? '' : $start		= $this->request->variable('start', 0);

			$default_sort_days	= !empty($this->user->data['user_post_show_days']) ? $this->user->data['user_post_show_days'] : 0;
			$default_sort_key	= !empty($this->user->data['user_post_sortby_type']) ? $this->user->data['user_post_sortby_type'] : 't';
			$default_sort_dir	= !empty($this->user->data['user_post_sortby_dir']) ? $this->user->data['user_post_sortby_dir'] : 'a';

			$sort_days	= $this->request->variable('st', $default_sort_days);
			$sort_key	= $this->request->variable('sk', $default_sort_key);
			$sort_dir	= $this->request->variable('sd', $default_sort_dir);
			if (
				$topic_id && !$view && !$update &&
				$sort_days == $default_sort_days &&
				$sort_key == $default_sort_key &&
				$sort_dir == $default_sort_dir)
			{
				$page_name="_cgbp_{$page}_t{$topic_id}_s{$start}";
			}
			else {
				return false;
			}
		}
		else {
			return false;
		}

		return $page_name;
	}

	public function is_cacheable_user($user_id, $is_bot)
	{
		if(
			($this->config['cgbp_enabled']==3 && $user_id != ANONYMOUS && !$is_bot)
				|| ($this->config['cgbp_enabled']==1 && $user_id != ANONYMOUS)
				|| ($this->config['cgbp_enabled']==2 && !$is_bot)
		)
		{
			return false;
		}

		return '_' . ($is_bot ? 'bot' : 'guest');
		//return '_' . $this->user->data['user_style'] . '_' . ($is_bot ? 'bot' : 'guest');
	}

	public function display_if_cached($cache_key, $is_bot)
	{
		// check for If-None-Match behavior
		$match = $this->request->server('HTTP_IF_NONE_MATCH') ? trim($this->request->server('HTTP_IF_NONE_MATCH')) : false;

		$cache_expired = $this->get_expiration($cache_key);

		$topic_id=0;
		if($this->config['cgbp_update_topic'])
		{
			preg_match('/^_cgbp_viewtopic_t(\d+)_/', $cache_key, $topic_id);
			isset($topic_id[1]) ? $topic_id=$topic_id[1] : '';
		}

		// is matches to cached file?
		if ($match !== false && $cache_expired !== false && $match == $cache_expired)
		{
			$topic_id ? $this->update_topic_info($topic_id) : '';

			// return not modified status
			send_status_line(304, 'Not Modified');

			$this->set_cache_headers($cache_expired, $is_bot);

			garbage_collection();
			exit_handler();
		}
		elseif (($cache_item = $this->cache->get($cache_key)) !== false)
		{
			$topic_id ? $this->update_topic_info($topic_id) : '';

			// gzip_compression (copied from page_header() )
			if ($this->config['gzip_compress'])
			{
				if (@extension_loaded('zlib') && !headers_sent() && ob_get_level() <= 1 && ob_get_length() == 0)
				{
					ob_start('ob_gzhandler');
				}
			}

			// not sure that we need for compatibility with HTTP Guest cache anymore
			// since own ETag/If-None-Match mechanism used now for cached pages
			$this->set_cache_headers($cache_expired, $is_bot);

			// echo debug information to the end of the page
			// it is not possible to use templated place, because no templates are used in cached output

			if($this->config['cgbp_debug_info'])
			{
				$debug_info = $this->generate_debug_info();

				$debug_info ? $cache_item = str_replace('<!-- CGBP DEBUG OUTPUT -->', '<div class="small" style="text-align:center;">'.implode(' | ', $debug_info).'</div>', $cache_item) : '';
			}

			// echo cached content
			echo $cache_item;

			unset($cache_item);

			garbage_collection();
			exit_handler();
		}

		// do nothing
	}

	public function cache_content_page($cache_key, $cache_ttl = 720)
	{
		// cache page content
		$cache_item = ob_get_contents();

		$expires = $this->cache->put($cache_key, $cache_item, $cache_ttl * 60);

		// override Cache-Control headers
		if (/*$this->config['cgbp_debug_info'] && */!headers_sent())
		{
			$this->set_cache_headers($expires, $this->user->data['is_bot']);
		}

		// note sure that we need to force buffer flushing,
		// anyway it is supposed that no more content operations will be performed.
		// Uncomment string below to force output buffer flushing
		//ob_end_flush();
	}

	public function set_cache_headers($etag, $is_bot = false)
	{
		// setting up page headers
		header('Content-type: text/html; charset=UTF-8');

		header('Cache-Control: public, no-cache="set-cookie"');
		header('Pragma: public');
		header('ETag: ' . $etag);

		if ($is_bot)
		{
			// Let reverse proxies know we detected a bot.
			header('X-PHPBB-IS-BOT: yes');
		}
	}

	public function get_expiration($var_name)
	{
		// works for 'data' cache only
		if ($var_name[0] != '_')
		{
			return false;
		}

		$filename = "{$this->cache_dir}data$var_name.{$this->phpEx}";

		if (!($handle = @fopen($filename, 'rb')))
		{
			return false;
		}

		// Skip the PHP header
		fgets($handle);

		// Get expiration
		$expires = (int) fgets($handle);

		fclose($handle);

		if (time() >= $expires)
		{
			$this->cache->remove_file($filename);
			return false;
		}
		else
		{
			return $expires;
		}
	}

	public function update_topic_info($topic_id)
	{
		// Update topic view counters ... but only for humans and if this is the first 'page view'
		if ($topic_id && isset($this->user->data['session_page']) && !$this->user->data['is_bot'] && (strpos($this->user->data['session_page'], '&t=' . $topic_id) === false || isset($this->user->data['session_created'])))
		{
			$sql = 'UPDATE ' . TOPICS_TABLE . '
				SET topic_views = topic_views + 1, topic_last_view_time = ' . time() . "
				WHERE topic_id = $topic_id";
			$this->db->sql_query($sql);
		}
	}

	public function destroy_topic_cache($topic_id, $topic_data = null, $destroy_forumcache=true, $destroy_indexcache=true)
	{
		// try to obtain $topic_data if it wasn't passed by param
		if (!$topic_data)
		{
			// I'd prefer to use same function from mcp.php,
			// but that file contains too much trash inside ;)
			$topic_data = $this->get_topic_data($topic_id);

			// can't obtain any topic information
			if ($topic_data === false)
			{
				return false;
			}
		}

		// destroy cache for topic's forum and forum's parents
		$destroy_forumcache ? $this->destroy_forum_cache($topic_data['forum_id'], $topic_data, $destroy_indexcache) : ($destroy_indexcache ? $this->destroy_index_cache() : '');

// 		$topic_data['topic_replies_real'] = $topic_data['topic_posts_approved'] + $topic_data['topic_posts_unapproved'] + $topic_data['topic_posts_softdeleted'] - 1;
		$topic_data['topic_replies'] = $topic_data['topic_posts_approved'] - 1;

		// calculate pages count
		$total_pages = ceil(($topic_data['topic_replies'] + 1) / $this->config['posts_per_page']);
		if(!isset($this->cgbp_exclude['viewtopic']) && !isset($this->topiccache_destroyed[$topic_id]))
		{
			// destroy cache for all topic pages, for both guest and bot
			for ($i = 0; $i < $total_pages; $i++)
			{
				$start = $i * $this->config['posts_per_page'];

				$this->cache->destroy("_cgbp_viewtopic_t{$topic_id}_s{$start}_guest");
				$this->cache->destroy("_cgbp_viewtopic_t{$topic_id}_s{$start}_bot");
			}
			$this->topiccache_destroyed[$topic_id]=true;
		}

		return true;
	}

	public function destroy_selected_cache($selected)
	{
		if(!$selected)
		{
			return false;
		}

		$selected_pattern='';
		foreach($selected as $type => $ary)
		{
			if($type=='viewforum')
			{
				$selected_pattern='^data_cgbp_viewforum_';
				if(!isset($ary['ids']))
				{
					$selected_pattern.='f\d+_';
				}
				else {
					$selected_pattern.='f(?:'.implode('|', $ary['ids']).')';
				}
			}
			else if($type=='viewtopic')
			{
				$selected_pattern='^data_cgbp_viewtopic_';
				if(!isset($ary['ids']))
				{
					$selected_pattern.='t\d+_';
				}
				else {
					$selected_pattern.='t(?:'.implode('|', $ary['ids']).')';
				}
			}
		}

		if(!$selected_pattern)
		{
			return false;
		}

		$dir = @opendir($this->cache_dir);

		if (!$dir)
		{
			return;
		}

		while (($entry = readdir($dir)) !== false)
		{
			if (strpos($entry, 'data_cgbp_')!==0)
			{
				continue;
			}

			if(preg_match('/'.$selected_pattern.'/', $entry))
			{
				@unlink($this->cache_dir . $entry);
			}

		}
		closedir($dir);
	}

	public function destroy_index_cache()
	{
		if(isset($this->cgbp_exclude['index']) || isset($this->indexcache_destroyed))
		{
			return false;
		}

		// destroy cache for index pages
		$this->cache->destroy("_cgbp_index_guest");
		$this->cache->destroy("_cgbp_index_bot");

		$this->indexcache_destroyed=true;
	}

	public function destroy_forum_cache($forum_id, $forum_data = null, $destroy_indexcache=true)
	{
		// try to obtain $forum_data if it wasn't passed by param
		if (!$forum_data)
		{
			// I'd prefer to use same function from mcp.php,
			// but that file contains too much trash inside ;)
			$forum_data = $this->get_forum_data($forum_id);

			// can't obtain any topic information
			if ($forum_data === false)
			{
				return false;
			}
		}

		// destroy cache for index pages
		$destroy_indexcache ? $this->destroy_index_cache() : '';
// 		$this->cache->destroy("_cgbp_index_guest");
// 		$this->cache->destroy("_cgbp_index_bot");

// 		$forum_data['forum_topics_real'] = $forum_data['forum_topics_approved'] + $forum_data['forum_topics_unapproved'] + $forum_data['forum_topics_softdeleted'] - 1;
		$forum_data['forum_topics'] = $forum_data['forum_topics_approved'] - 1;

		// calculate pages count
		$total_pages = ceil(($forum_data['forum_topics'] + 1) / $this->config['topics_per_page']);
		if(!isset($this->cgbp_exclude['viewforum']) && !isset($this->forumcache_destroyed[$forum_id]))
		{
			// destroy cache for all topic pages, for both guest and bot
			for ($i = 0; $i < $total_pages; $i++)
			{
				$start = $i * $this->config['topics_per_page'];

				$this->cache->destroy("_cgbp_viewforum_f{$forum_data['forum_id']}_s{$start}_guest");
				$this->cache->destroy("_cgbp_viewforum_f{$forum_data['forum_id']}_s{$start}_bot");
			}
			$this->forumcache_destroyed[$forum_id]=true;
		}

		if(!isset($this->cgbp_exclude['viewforum']))
		{
			// destroy cache for forum's parents as well
			if (!function_exists('get_forum_parents'))
			{
				include($this->phpbb_root_path . 'includes/functions_display.' . $this->phpEx);
			}

			$forum_parents = get_forum_parents($forum_data);
			foreach ($forum_parents as $forum_id => $value)
			{
				if(isset($this->forumcache_destroyed[$forum_id]))
				{
					continue;
				}
				// Honestly, we need calculate amount of pages for each of these underlying forums.
				// But I'm not sure that heap of additional sql requests are appropriated here.
				// I assume outdated information related to latest message in child forum
				// doesn't costs any additional sql request. For guests, of course. Register for realtime data, yah! :)
				$this->cache->destroy("_cgbp_viewforum_f{$forum_id}_s0_guest");
				$this->cache->destroy("_cgbp_viewforum_f{$forum_id}_s0_bot");

				$this->forumcache_destroyed[$forum_id]=true;
			}
		}

		return true;
	}

	public function get_topic_data($topic_id)
	{
		$topic_data=array();

		// get related forum data to clear corresponding forum caches as well
		$sql_array = array(
			'SELECT' => 't.*, f.*',

			'FROM' => array(FORUMS_TABLE => 'f'),
			);

		// Firebird handles two columns of the same name a little differently, this
		// addresses that by forcing the forum_id to come from the forums table.
// 		if ($this->db->get_sql_layer === 'firebird')
// 		{
// 			$sql_array['SELECT'] = 'f.forum_id AS forum_id, ' . $sql_array['SELECT'];
// 		}

		// Topics table need to be the last in the chain
		$sql_array['FROM'][TOPICS_TABLE] = 't';

		$sql_array['WHERE'] = $this->db->sql_in_set('t.topic_id', $topic_id);

		$sql_array['WHERE'] .= ' AND (f.forum_id = t.forum_id';

		// If it is a global announcement make sure to set the forum id to a postable forum
		$sql_array['WHERE'] .= ' OR (t.topic_type = ' . POST_GLOBAL . '
			AND f.forum_type = ' . FORUM_POST . ')';

		$sql_array['WHERE'] .= ')';

		// Join to forum table on topic forum_id unless topic forum_id is zero
		// whereupon we join on the forum_id passed as a parameter ... this
		// is done so navigation, forum name, etc. remain consistent with where
		// user clicked to view a global topic
		$sql = $this->db->sql_build_query('SELECT', $sql_array);
		$result = $this->db->sql_query($sql);
		while($row = $this->db->sql_fetchrow($result))
		{
			$topic_data[$row['topic_id']]=$row;
		}
		$this->db->sql_freeresult($result);

		if (!$topic_data)
		{
			return false;
		}
		else
		{
			return is_array($topic_id) ? $topic_data : $topic_data[$topic_id];
		}
	}

	public function get_forum_data($forum_id)
	{
		$forum_data=array();

		$sql = "SELECT *
			FROM " . FORUMS_TABLE .
			" WHERE " . $this->db->sql_in_set('forum_id', $forum_id);
		$result = $this->db->sql_query($sql);
		while($row = $this->db->sql_fetchrow($result))
		{
			$forum_data[$row['forum_id']]=$row;
		}
		$this->db->sql_freeresult($result);

		if (!$forum_data)
		{
			return false;
		}
		else
		{
			return is_array($forum_id) ? $forum_data : $forum_data[$forum_id];
		}
	}

	public function generate_debug_info()
	{
		$debug_info = array();

		// Output page creation time
		if (defined('PHPBB_DISPLAY_LOAD_TIME'))
		{
			if (isset($GLOBALS['starttime']))
			{
				$totaltime = microtime(true) - $GLOBALS['starttime'];
				$debug_info[] = sprintf('<span>CGBP Time: %.3fs</span>', $totaltime);
			}

			$debug_info[] = sprintf('<span>Queries: %d</span>', $this->db->sql_num_queries());

			$memory_usage = memory_get_peak_usage();
			if ($memory_usage)
			{
				$memory_usage = get_formatted_filesize($memory_usage);

				$debug_info[] = 'Peak Memory Usage: ' . $memory_usage;
			}
		}

		if (defined('DEBUG'))
		{
			$debug_info[] = 'GZIP: ' . (($this->config['gzip_compress'] && @extension_loaded('zlib')) ? 'On' : 'Off');

			if ($this->user->load)
			{
				$debug_info[] = 'Load: ' . $this->user->load;
			}
		}

		return $debug_info;
	}
}

?>
