<?php

/**
 *
 * @package CacheGBPages
 * @copyright (c) 2024 PPK
 * @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
 *
 *
 */

namespace ppk\cachegbpages\cron\task;

class cgbp_task extends \phpbb\cron\task\base
{
	protected $config;
	protected $phpbb_root_path;
	protected $php_ext;

	public $dt;
	public $cache_dir;

	public function __construct(\phpbb\config\config $config, $phpbb_root_path, $php_ext)
	{
		$this->config = $config;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->phpEx = $php_ext;

		$this->dt = time();
		$this->cache_dir = $phpbb_root_path . 'cache/'.PHPBB_ENVIRONMENT.'/';

	}

	/**
	* Runs this cron task.
	*
	* @return null
	*/
	public function run()
	{
		$dir = @opendir($this->cache_dir);

		if (!$dir)
		{
			return;
		}

		while (($entry = readdir($dir)) !== false)
		{
			if (strpos($entry, 'data_cgbp_')!==0)
			{
				continue;
			}

			if (!($handle = @fopen($this->cache_dir . $entry, 'rb')))
			{
				continue;
			}

			// Skip the PHP header
			fgets($handle);

			// Skip expiration
			$expires = (int) fgets($handle);

			fclose($handle);

			if ($this->dt >= $expires)
			{
				@unlink($this->cache_dir . $entry);
			}
		}
		closedir($dir);

		// Do not forget to update the configuration variable for last run time.
		$this->config->set('cgbp_cache_last_gc', $this->dt, true);
	}

	/**
	* Returns whether this cron task should run now, because enough time
	* has passed since it was last run.
	*
	* @return bool
	*/
	public function should_run()
	{
		return $this->config['cgbp_cache_gc'] && $this->dt - $this->config['cgbp_cache_last_gc'] > $this->config['cgbp_cache_gc'] * 60 ? true : false;
	}

	public function is_runnable()
	{
		return true;
	}

}