<?php

/**
*
* @package CacheGBPages
* @copyright (c) 2024 PPK
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*
*/

namespace ppk\cachegbpages\event;

/**
 * @ignore
 */
if (!defined('IN_PHPBB'))
{
	exit;
}

use Symfony\Component\EventDispatcher\EventSubscriberInterface;

class listener implements EventSubscriberInterface
{

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\user */
	protected $user;

	/** @var scalar string */
	protected $phpbb_root_path;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var string PHP extension */
	protected $phpEx;

	/** @var \phpbb\cache\service */
	protected $cache;

	/* @var \ppk\cachegbpages\core\cgbp_functions */
	protected $cgbp_functions;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	public function __construct(\phpbb\template\template $template, \phpbb\config\config $config, \phpbb\user $user, $phpbb_root_path, \phpbb\request\request $request, $phpEx, \phpbb\cache\service $cache, \ppk\cachegbpages\core\cgbp_functions $functions, \phpbb\db\driver\driver_interface $db)
	{
		$this->template = $template;
		$this->config = $config;
		$this->user = $user;
		$this->phpbb_root_path = $phpbb_root_path;
		$this->request = $request;
		$this->phpEx = $phpEx;
		$this->cache = $cache;
		$this->cgbp_functions = $functions;
		$this->db = $db;

		$this->run_cron=false;
		$this->cache_dir = $phpbb_root_path . 'cache/'.PHPBB_ENVIRONMENT.'/';

	}

	static public function getSubscribedEvents()
	{
		return array(
			'core.user_setup' => 'cgbp_main',
			'core.page_header' => 'cgbp_gzcompress',
			'core.page_footer' => 'cgbp_getcron',
			'core.page_footer_after' => 'cgbp_cachepage',
			'core.append_sid' => 'cgbp_remove_sid',
			'core.viewtopic_before_f_read_check' => 'cgbp_viewtopic',
			'core.posting_modify_submit_post_after' => 'cgbp_submit_post',
			'core.move_topics_before' => 'cgbp_move_topics',
			'core.move_posts_before' => 'cgbp_move_posts',
			'core.delete_topics_before_query' => 'cgbp_delete_topics',
			'core.delete_posts_in_transaction_before' => 'cgbp_delete_posts',
			'core.mcp_lock_unlock_after' => 'cgbp_lock_unlock',
			'core.mcp_change_topic_type_before' => 'cgbp_change_topic_type',
			'core.mcp_main_modify_fork_sql' => 'cgbp_fork_topic',

		);
	}

	public function cgbp_fork_topic($event)
	{
		if(!$this->config['cgbp_enabled'] || isset($this->fork_action))
		{
			return false;
		}

		$sql_ary=$event['sql_ary'];
		$forum_id=$this->request->variable('f', 0);
		$forum_data=$this->cgbp_functions->get_forum_data(array_unique(array($forum_id, $sql_ary['forum_id'])));

		$this->cgbp_functions->destroy_forum_cache($sql_ary['forum_id'], $forum_data[$sql_ary['forum_id']]);
		if($forum_id!=$sql_ary['forum_id'])
		{
			$this->cgbp_functions->destroy_forum_cache($forum_id, $forum_data[$forum_id]);
		}

		$this->fork_action=true;
	}

	public function cgbp_change_topic_type($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$topic_ids=$event['topic_ids'];

		if (count($topic_ids))
		{
			$data = $this->cgbp_functions->get_topic_data($topic_ids);
			$new_topic_type=$event['new_topic_type'];

			$destroy_all_fcache=$new_topic_type==POST_GLOBAL ? 1 : 0;
			foreach ($data as $topic_id => $row)
			{
				if($row['topic_type']==POST_GLOBAL)
				{
					$destroy_all_fcache+=1;
				}
				$this->cgbp_functions->destroy_topic_cache($topic_id, $row, true, false);
			}

			if($destroy_all_fcache)
			{
				$this->cgbp_functions->destroy_selected_cache(array('viewforum'=>array()));
			}
		}
	}

	public function cgbp_lock_unlock($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$ids=$event['ids'];
		$action=$event['action'];
		$data=$event['data'];

		if ($action == 'lock' || $action == 'unlock')
		{
			foreach ($ids as $t_id)
			{
				$this->cgbp_functions->destroy_topic_cache($t_id, $data[$t_id], true, false);
			}
		}
	}

	public function cgbp_delete_posts($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$topic_ids=$event['topic_ids'];
		$topic_data=$this->cgbp_functions->get_topic_data($topic_ids);

		foreach ($topic_data as $t_id => $row)
		{
			$this->cgbp_functions->destroy_topic_cache($t_id, $row);
		}
	}

	public function cgbp_move_posts($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$topic_ids=$event['topic_ids'];
		$topic_data=$this->cgbp_functions->get_topic_data($topic_ids);

		foreach ($topic_data as $t_id => $row)
		{
			$this->cgbp_functions->destroy_topic_cache($t_id, $row);
		}
	}

	public function cgbp_move_topics($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$topic_ids=$event['topic_ids'];
		$topic_data=$this->cgbp_functions->get_topic_data($topic_ids);

		foreach ($topic_data as $t_id => $row)
		{
			$this->cgbp_functions->destroy_topic_cache($t_id, $row);
		}

		$forum_ids=array($event['forum_id']);
		$forum_data=$this->cgbp_functions->get_forum_data($forum_ids);
		foreach ($forum_data as $f_id => $row)
		{
			$this->cgbp_functions->destroy_forum_cache($f_id, $row);
		}
	}

	public function cgbp_delete_topics($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$topic_ids=$event['topic_ids'];
		$topic_data=$this->cgbp_functions->get_topic_data($topic_ids);

		foreach ($topic_data as $t_id => $row)
		{
			$this->cgbp_functions->destroy_topic_cache($t_id, $row);
		}

		$forum_ids=array();
		$sql = 'SELECT forum_id
			FROM ' . TOPICS_TABLE . '
			WHERE ' . $this->db->sql_in_set('topic_moved_id', $topic_ids);
		$result = $this->db->sql_query($sql);
		while ($row = $this->db->sql_fetchrow($result))
		{
			$forum_ids[] = $row['forum_id'];
		}
		$this->db->sql_freeresult($result);

		if($forum_ids)
		{
			$forum_data=$this->cgbp_functions->get_forum_data($forum_ids);
			foreach ($forum_data as $f_id => $row)
			{
				$this->cgbp_functions->destroy_forum_cache($f_id, $row);
			}
		}

	}

	public function cgbp_submit_post($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$topic_id=$event['topic_id'];
		$post_data=$event['post_data'];

		$this->cgbp_functions->destroy_topic_cache($topic_id, $post_data);

	}

	public function cgbp_viewtopic($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$user_type_suffix=$this->cgbp_functions->is_cacheable_user($this->user->data['user_id'], $this->user->data['is_bot']);
		if (!$user_type_suffix)
		{
			return false;
		}

		$topic_data=$event['topic_data'];
		$post_id=$event['post_id'];
		$start=0;
		if ($post_id)
		{
			$start = floor(($topic_data['prev_posts']) / $this->config['posts_per_page']) * $this->config['posts_per_page'];
		}

		$page_name=$this->cgbp_functions->is_cacheable_page($this->request->server('SCRIPT_NAME'), $topic_data['topic_id'], $start);
		if(!$page_name)
		{
			return false;
		}

		define('CGBP_KEY', $page_name . $user_type_suffix);

		$this->cgbp_functions->display_if_cached(CGBP_KEY, $this->user->data['is_bot']);
	}

	public function cgbp_remove_sid($event)
	{
		if(!$this->config['cgbp_remove_sid'])
		{
			return false;
		}

		if(defined('CGBP_KEY'))
		{
			$session_id=$event['session_id'];
			$session_id = '';

			$event['session_id']=$session_id;
		}
	}

	public function cgbp_getcron($event)
	{
		$this->run_cron=$event['run_cron'];
	}

	public function cgbp_cachepage($event)
	{
		if (defined('CGBP_KEY'))
		{
			$display_template=$event['display_template'];
			if (!$display_template)
			{
				return false;
			}

			$call_cron = false;
			if (!defined('IN_CRON') && !$this->config['use_system_cron'] && $this->run_cron && !$this->config['board_disable'] && !$this->user->data['is_bot'] && !$this->cache->get('_cron.lock_check'))
			{
				$call_cron = true;
				$time_now = (!empty($this->user->time_now) && is_int($this->user->time_now)) ? $this->user->time_now : time();

				// Any old lock present?
				if (!empty($this->config['cron_lock']))
				{
					$cron_time = explode(' ', $this->config['cron_lock']);

					// If 1 hour lock is present we do not call cron.php
					if ($cron_time[0] + 3600 >= $time_now)
					{
						$call_cron = false;
					}
				}
			}
			// don't cache pages with cron calls
			if($call_cron)
			{
				return false;
			}

			$this->template->display('body');

			$this->cgbp_functions->cache_content_page(CGBP_KEY, $this->config['cgbp_ttl']);

			$display_template=false;
			$event['display_template']=$display_template;
		}
	}

	public function cgbp_gzcompress()
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		if(!$this->config['gzip_compress'])
		{
			if (defined('CGBP_KEY') && !headers_sent() && ob_get_level() <= 1 && ob_get_length() == 0)
			{
				// we have to use output buffer for saving cached content
				ob_start();
			}
		}
	}

	public function cgbp_main($event)
	{
		if(!$this->config['cgbp_enabled'])
		{
			return false;
		}

		$lang_set=$event['lang_set'];
		if(is_array($lang_set) || !in_array($lang_set, array(/*'index', */'viewforum'/*, 'viewtopic'*/)))
		{
			return false;
		}

		$user_type_suffix=$this->cgbp_functions->is_cacheable_user($this->user->data['user_id'], $this->user->data['is_bot']);
		if (!$user_type_suffix)
		{
			return false;
		}

		$page_name=$this->cgbp_functions->is_cacheable_page($this->request->server('SCRIPT_NAME'));
		if(!$page_name)
		{
			return false;
		}

		define('CGBP_KEY', $page_name . $user_type_suffix);

		$this->cgbp_functions->display_if_cached(CGBP_KEY, $this->user->data['is_bot']);

	}


}
